﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;
using MotorPositionController_Example.Resources;

namespace MotorPositionController_Example {
	public partial class Form1 : Form {
		CommandLineOpen open;
		MotorPositionController motor = null;
		private ErrorEventBox errorBox = null;
		private SettingsForm settingsBox = null;
		public struct defaultMotorSettings {
			public double Kp;
			public double Ki;
			public double Kd;
			public double acceleration;
			public double velocityLimit;
			public double rescaleFactor;
			public double deadband;
			public double currentLimit;
			public double currentRegulatorGain;
			public double stallVelocity;
			public EncoderIOMode encoderIOMode;
		}

		defaultMotorSettings defaults;

		private int dataPoints;

		public Form1() {
			open = new CommandLineOpen(this);
			InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
			open.commandLine = commandLine;
			InitializeComponent();
		}

		#region FormEvents
		private void Form1_Load(object sender, EventArgs e) {
			
			defaults = new defaultMotorSettings();
			errorBox = new ErrorEventBox(this);
			settingsBox = new SettingsForm(this);
			PositionChart.ChartAreas[0].AxisX.ScrollBar.Enabled = false;

			motor = new MotorPositionController();
			motor.Attach += motor_attach;
			motor.Detach += motor_detach;
			motor.Error += motor_error;
			motor.PositionChange += motor_position_change;
			motor.DutyCycleUpdate += motor_dutycycle_update;

            commandLineData phidgetParameters = open.parseCmdLine(); //get command line parameters

            try
            { //set all the values grabbed from command line.  these values have defaults that are set in ExampleUtils.cs, you can check there to see them.
                motor.Channel = phidgetParameters.Channel; //selects the channel on the device to open
                motor.DeviceSerialNumber = phidgetParameters.SerialNumber; //selects the device or hub to open
                motor.HubPort = phidgetParameters.HubPort; //selects the port on the hub to open
                motor.IsHubPortDevice = phidgetParameters.isHubPortDevice; //is the device a port on a VINT hub?

                if (phidgetParameters.isRemote) //are we trying to open a remote device?
                {
                    motor.IsRemote = true;
                    Net.EnableServerDiscovery(ServerType.Device); //turn on network scan
                    if (phidgetParameters.Password != null && phidgetParameters.ServerName != null)
                        Net.SetServerPassword(phidgetParameters.ServerName, phidgetParameters.Password); //set the password if there is one
                }
                else
                    motor.IsLocal = true;

                motor.Open(); //open the device specified by the above parameters
            }
            catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
        }

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
			motor.Attach -= motor_attach;
			motor.Detach -= motor_detach;
			motor.Error -= motor_error;
			motor.PositionChange -= motor_position_change;
			motor.DutyCycleUpdate -= motor_dutycycle_update;
			motor.Close();
		}
		#endregion

		#region motor events/updates
		void motor_attach(object sender, AttachEventArgs e) {
			MotorPositionController attached = (MotorPositionController)sender;
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			//Grab default values
			try {
				defaults.Kp = attached.Kp;
				defaults.Ki = attached.Ki;
				defaults.Kd = attached.Kd;
				defaults.acceleration = attached.Acceleration;
				defaults.velocityLimit = attached.VelocityLimit;
				defaults.rescaleFactor = attached.RescaleFactor;
				defaults.deadband = attached.DeadBand;
				if (attached.DeviceID == DeviceID.PN_DCC1000 || attached.DeviceID == DeviceID.PN_DCC1001)
					defaults.currentLimit = attached.CurrentLimit;
				if (attached.DeviceID == DeviceID.PN_DCC1000) {
					defaults.currentRegulatorGain = attached.CurrentRegulatorGain;
					defaults.encoderIOMode = attached.IOMode;
				}
				if (attached.DeviceID == DeviceID.PN_DCC1100)
					defaults.stallVelocity = attached.StallVelocity;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error getting device defaults: " + ex.Message);
			}


			//change data interval
			attached.DataInterval = attached.MinDataInterval;

			//Set up "other motor settings" box, which is intially hidden
			switch (attached.DeviceID) {
			case DeviceID.PN_DCC1000:
				currentLimPanel.Visible = true;
				currentRegPanel.Visible = true;
				ioModePanel.Visible = true;
				fanModePanel.Visible = true;
				currentLimitText.Minimum = Convert.ToDecimal(attached.MinCurrentLimit);
				currentLimitText.Maximum = Convert.ToDecimal(attached.MaxCurrentLimit);
				currentLimitText.Value = Convert.ToDecimal(attached.CurrentLimit);
				currentRegText.Value = Convert.ToDecimal(attached.CurrentRegulatorGain);
				foreach (EncoderIOMode eio in Enum.GetValues(typeof(EncoderIOMode)))
					encoderIOCombo.Items.Add(eio);
				encoderIOCombo.SelectedIndex = 0;

				foreach (FanMode fm in Enum.GetValues(typeof(FanMode)))
					fanModeCombo.Items.Add(fm);
				fanModeCombo.SelectedIndex = 2;

				additionalSettings.Controls.SetChildIndex(currentLimPanel, 0);
				additionalSettings.Controls.SetChildIndex(currentRegPanel, 1);
				additionalSettings.Controls.SetChildIndex(ioModePanel, 2);
				additionalSettings.Controls.SetChildIndex(fanModePanel, 3);
				break;
			case DeviceID.PN_DCC1001:
				currentLimPanel.Visible = true;
				currentLimitText.Minimum = Convert.ToDecimal(attached.MinCurrentLimit);
				currentLimitText.Maximum = Convert.ToDecimal(attached.MaxCurrentLimit);
				currentLimitText.Value = Convert.ToDecimal(attached.CurrentLimit);

				additionalSettings.Controls.SetChildIndex(currentLimPanel, 0);
				break;
			case DeviceID.PN_DCC1100:
				stallVelocityPanel.Visible = true;
				stallVelocityText.Value = Convert.ToDecimal(attached.StallVelocity);

				additionalSettings.Controls.SetChildIndex(stallVelocityPanel, 0);
				break;
			}

            //Try to set saved settings
            applySavedSettings(attached, null);
        }



		void motor_detach(object sender, DetachEventArgs e) {
			phidgetInfoBox1.Clear();
			clearGraphPoints();
			currentLimPanel.Visible = false;
			currentRegPanel.Visible = false;
			ioModePanel.Visible = false;
			fanModePanel.Visible = false;
			stallVelocityPanel.Visible = false;
			setEngaged.Text = "Engage";
			kpText.Text = "0";
			kiText.Text = "0";
			kdText.Text = "0";
			targetPositionText.Text = "0";
			accelerationText.Text = "0";
			velocityText.Text = "0";
			rescaleText.Text = "?";
			deadBandText.Text = "?";
			positionLabel.Text = "0";
			dutyCycleLabel.Text = "0";
		}

		void motor_error(object sender, ErrorEventArgs e) {
			errorBox.addMessage("Error " + e.Code + ": " + e.Description);
			if (e.Code == ErrorEventCode.MotorStallCondition) //this error will disengage the motor
			{
				setEngaged.Text = "Engage Motor";
			}
		}

		void motor_position_change(object sender, MotorPositionControllerPositionChangeEventArgs e) {
			PositionChart.Series["MotorPosition"].Points.AddY(e.Position);
			PositionChart.Series["TargetPosition"].Points.AddY(motor.TargetPosition);
			positionLabel.Text = e.Position.ToString("F3");
			while (PositionChart.Series["MotorPosition"].Points.Count > dataPoints) {
				PositionChart.Series["MotorPosition"].Points.RemoveAt(0);
				PositionChart.Series["TargetPosition"].Points.RemoveAt(0);
			}
			PositionChart.ChartAreas[0].RecalculateAxesScale();
		}

		void motor_dutycycle_update(object sender, MotorPositionControllerDutyCycleUpdateEventArgs e) {
			dutyCycleLabel.Text = e.DutyCycle.ToString();
		}
		#endregion

		#region graph/misc functions

		void updateRescaleProperties(MotorPositionController ch) {
			accelerationText.Increment = updateLimits(ch.Acceleration);
			velocityText.Increment = updateLimits(ch.VelocityLimit);
			targetPositionText.Minimum = Convert.ToDecimal(ch.MinPosition);
			targetPositionText.Maximum = Convert.ToDecimal(ch.MaxPosition);
			if (ch.DeviceID == DeviceID.PN_DCC1100) {
				stallVelocityText.Increment = updateLimits(ch.StallVelocity);
				stallVelocityText.Text = ch.StallVelocity.ToString();
			}

			accelerationText.Text = ch.Acceleration.ToString();
			velocityText.Text = ch.VelocityLimit.ToString();
			deadBandText.Text = ch.DeadBand.ToString();
		}

		void updateGUI(MotorPositionController ch) {
			//Fill in form with info from device
			kpText.Value = Convert.ToDecimal(ch.Kp);
			kiText.Value = Convert.ToDecimal(ch.Ki);
			kdText.Value = Convert.ToDecimal(ch.Kd);
			rescaleText.Text = ch.RescaleFactor.ToString();
			if (ch.DeviceID == DeviceID.PN_DCC1000 || ch.DeviceID == DeviceID.PN_DCC1001)
				currentLimitText.Value = Convert.ToDecimal(ch.CurrentLimit);
			if (ch.DeviceID == DeviceID.PN_DCC1000)
				currentRegText.Value = Convert.ToDecimal(ch.CurrentRegulatorGain);
			if (ch.DeviceID == DeviceID.PN_DCC1100)
				stallVelocityText.Value = Convert.ToDecimal(ch.StallVelocity);
			updateRescaleProperties(ch);

			//Set Numeric UpDown Limits
			kpText.Increment = updateLimits(ch.Kp);
			kdText.Increment = updateLimits(ch.Kd);
			kiText.Increment = updateLimits(ch.Ki);
		}

		void applySavedSettings(MotorPositionController ch, CheckedListBox.CheckedItemCollection collection) {
			if (collection == null) {
				try {
					//Set rescale because it will impact other properties
					if (Properties.Settings.Default.isRescaleFactorSet)
						ch.RescaleFactor = Properties.Settings.Default.rescaleFactor;

					if (Properties.Settings.Default.isKpSet)
						ch.Kp = Properties.Settings.Default.Kp;

					if (Properties.Settings.Default.isKiSet)
						ch.Ki = Properties.Settings.Default.Ki;

					if (Properties.Settings.Default.isKdSet)
						ch.Kd = Properties.Settings.Default.Kd;

					if (Properties.Settings.Default.isAccelerationSet)
						ch.Acceleration = Properties.Settings.Default.Acceleration;

					if (Properties.Settings.Default.isVelocityLimitSet)
						ch.VelocityLimit = Properties.Settings.Default.VelocityLimit;

					if (Properties.Settings.Default.isDeadbandSet)
						ch.DeadBand = Properties.Settings.Default.Deadband;

					if (Properties.Settings.Default.isCurrentLimitSet && (ch.DeviceID == DeviceID.PN_DCC1000 || ch.DeviceID == DeviceID.PN_DCC1001))
						ch.CurrentLimit = Properties.Settings.Default.CurrentLimit;

					if (Properties.Settings.Default.isEncoderIOModeSet && (ch.DeviceID == DeviceID.PN_DCC1000))
						ch.IOMode = Properties.Settings.Default.encoderIOMode;

					if (Properties.Settings.Default.isCurrentRegulatorGainSet && ch.DeviceID == DeviceID.PN_DCC1000)
						ch.CurrentRegulatorGain = Properties.Settings.Default.CurrentRegulatorGain;

					if (Properties.Settings.Default.isStallVelocitySet && ch.DeviceID == DeviceID.PN_DCC1100)
						ch.StallVelocity = Properties.Settings.Default.StallVelocity;

					if (Properties.Settings.Default.isDataPointsSet) {
						dataPoints = Properties.Settings.Default.dataPoints;
						dataPointsText.Text = dataPoints.ToString();
					} else {
						dataPoints = 100;
						dataPointsText.Text = "100";
					}
				} catch (PhidgetException ex) {
					errorBox.addMessage("Error setting saved parameter: " + ex.Message);
				}
			} else {
				try {
					for (int i = 0; i < collection.Count; i++) {
						if (collection[i].ToString() == "Kp") {
							ch.Kp = Properties.Settings.Default.Kp;
						} else if (collection[i].ToString() == "Ki") {
							ch.Ki = Properties.Settings.Default.Ki;
						} else if (collection[i].ToString() == "Kd") {
							ch.Kd = Properties.Settings.Default.Kd;
						} else if (collection[i].ToString() == "Acceleration") {
							ch.Acceleration = Properties.Settings.Default.Acceleration;
						} else if (collection[i].ToString() == "Velocity Limit") {
							ch.VelocityLimit = Properties.Settings.Default.VelocityLimit;
						} else if (collection[i].ToString() == "Rescale Factor") {
							ch.RescaleFactor = Properties.Settings.Default.rescaleFactor;
						} else if (collection[i].ToString() == "Deadband") {
							ch.DeadBand = Properties.Settings.Default.Deadband;
						} else if (collection[i].ToString() == "Current Limit") {
							ch.CurrentLimit = Properties.Settings.Default.CurrentLimit;
						} else if (collection[i].ToString() == "Current Regulator Gain") {
							ch.CurrentRegulatorGain = Properties.Settings.Default.CurrentRegulatorGain;
						} else if (collection[i].ToString() == "Stall Velocity") {
							ch.StallVelocity = Properties.Settings.Default.StallVelocity;
						}
					}
				} catch (PhidgetException ex) {
					errorBox.addMessage("Error getting default values: " + ex.Message);
				}
			}

			updateGUI(ch);
		}

		void setDefaults(MotorPositionController ch, CheckedListBox.CheckedItemCollection collection) {
			try {
				for (int i = 0; i < collection.Count; i++) {
					if (collection[i].ToString() == "Kp") {
						ch.Kp = defaults.Kp;
					} else if (collection[i].ToString() == "Ki") {
						ch.Ki = defaults.Ki;
					} else if (collection[i].ToString() == "Kd") {
						ch.Kd = defaults.Kd;
					} else if (collection[i].ToString() == "Acceleration") {
						ch.Acceleration = defaults.acceleration;
					} else if (collection[i].ToString() == "Velocity Limit") {
						ch.VelocityLimit = defaults.velocityLimit;
					} else if (collection[i].ToString() == "Rescale Factor") {
						ch.RescaleFactor = defaults.rescaleFactor;
					} else if (collection[i].ToString() == "Deadband") {
						ch.DeadBand = defaults.deadband;
					} else if (collection[i].ToString() == "Current Limit") {
						ch.CurrentLimit = defaults.currentLimit;
					} else if (collection[i].ToString() == "Current Regulator Gain") {
						ch.CurrentRegulatorGain = defaults.currentRegulatorGain;
					} else if (collection[i].ToString() == "Stall Velocity") {
						ch.StallVelocity = defaults.stallVelocity;
					} else if (collection[i].ToString() == "Encoder IO Mode") {
						ch.IOMode = defaults.encoderIOMode;
					}
				}
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting default values: " + ex.Message);
			}

			updateGUI(ch);
		}

		private Decimal updateLimits(double n) {
			n = Math.Abs(n);
			if (n != 0 && n > 1)
				return Convert.ToDecimal(Math.Pow(10, (Math.Floor(Math.Log10(n)))) / 2);
			else
				return Convert.ToDecimal(0.5);
		}

		private void clearGraphPoints() {
			foreach (var series in PositionChart.Series) {
				series.Points.Clear();
			}
		}

		private void clearGraph_Click(object sender, EventArgs e) {
			clearGraphPoints();
		}
		#endregion

		#region GUI controls
		private void setRescaleFactor_Click(object sender, EventArgs e) {
			double result;
			if (double.TryParse(rescaleText.Text, out result)) {
				try {
					motor.RescaleFactor = result;
				} catch (PhidgetException ex) {
					errorBox.addMessage("Error setting rescale factor: " + ex.Message);
				}
			}
			updateRescaleProperties(motor);
		}

		private void setDeadband_Click(object sender, EventArgs e) {
			double result;
			if (double.TryParse(deadBandText.Text, out result)) {
				try {
					motor.DeadBand = result;
				} catch (PhidgetException ex) {
					errorBox.addMessage("Error setting deadband: " + ex.Message);
				}
			}
		}

		private void setEngaged_Click(object sender, EventArgs e) {
			try {
				if (motor.Engaged) {
					motor.Engaged = false;
					setEngaged.Text = "Engage Motor";
				} else {
					motor.Engaged = true;
					setEngaged.Text = "Disengage Motor";
				}
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting engaged: " + ex.Message);
			}
		}

		private void zeroPosition_Click(object sender, EventArgs e) {
			try {
				motor.AddPositionOffset(-1 * motor.Position);
				//clear graph
				clearGraphPoints();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error zeroing position" + ex.Message);
			}


		}

		#region targetPosition
		private void targetTextUpdate(object sender, EventArgs e) {
			incrementPosition();
		}

		private void incrementPositionText_KeyDown(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				e.SuppressKeyPress = true;
				incrementPosition();
			}
		}

		private void incrementPosition() {
			double result;
			if (double.TryParse(incrementPositionText.Text, out result)) {
				if (result > 0) //can't have negative increment value
					targetPositionText.Increment = Convert.ToDecimal(result);
			}
		}


		private void targetPositionText_ValueChanged(object sender, EventArgs e) {
			if (motor.Attached) //prevent text change event from happening when resetting textbox on detach
				setTargetPosition();
		}

		private void targetPositionText_KeyDown(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				e.SuppressKeyPress = true;
				setTargetPosition();
			}
		}

		private void setTargetPosition() {
			try {
				motor.TargetPosition = Convert.ToDouble(targetPositionText.Value);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting target position: " + ex.Message);
			}
		}
		#endregion

		#region velocity
		private void velocityText_ValueChanged(object sender, EventArgs e) {
			if (motor.Attached) //prevent text change event from happening when resetting textbox on detach
				setVelocity();
		}

		private void velocityText_KeyDown(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				e.SuppressKeyPress = true;
				setVelocity();
			}
		}

		private void setVelocity() {
			try {
				motor.VelocityLimit = Convert.ToDouble(velocityText.Value);
				velocityText.Increment = updateLimits(motor.VelocityLimit);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting velocity: " + ex.Message);
			}
		}
		#endregion

		#region acceleration
		private void accelerationText_ValueChanged(object sender, EventArgs e) {
			if (motor.Attached) //prevent text change event from happening when resetting textbox on detach
				setAcceleration();
		}

		private void accelerationText_KeyDown(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				e.SuppressKeyPress = true;
				setAcceleration();
			}
		}

		private void setAcceleration() {
			try {
				motor.Acceleration = Convert.ToDouble(accelerationText.Value);
				accelerationText.Increment = updateLimits(motor.Acceleration);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting acceleration: " + ex.Message);
			}
		}
		#endregion

		#region Kp
		private void kpText_ValueChanged(object sender, EventArgs e) {
			if (motor.Attached) //prevent text change event from happening when resetting textbox on detach
				setKp();
		}

		private void kpText_KeyDown(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				e.SuppressKeyPress = true;
				setKp();
			}
		}

		private void setKp() {
			try {
				motor.Kp = Convert.ToDouble(kpText.Value);
				kpText.Increment = updateLimits(motor.Kp);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting Ki: " + ex.Message);
			}
		}
		#endregion

		#region Ki
		private void kiText_ValueChanged(object sender, EventArgs e) {
			if (motor.Attached) //prevent text change event from happening when resetting textbox on detach
				setKi();
		}

		private void kiText_KeyDown(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				e.SuppressKeyPress = true;
				setKi();
			}
		}

		private void setKi() {
			try {
				motor.Ki = Convert.ToDouble(kiText.Value);
				kiText.Increment = updateLimits(motor.Ki);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting Ki: " + ex.Message);
			}
		}
		#endregion

		#region Kd
		private void kdText_ValueChanged(object sender, EventArgs e) {
			if (motor.Attached) //prevent text change event from happening when resetting textbox on detach
				setKd();
		}

		private void kdText_KeyDown(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				e.SuppressKeyPress = true;
				setKd();
			}
		}

		private void setKd() {
			try {
				motor.Kd = Convert.ToDouble(kdText.Value);
				kdText.Increment = updateLimits(motor.Kd);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting Kd: " + ex.Message);
			}
		}
		#endregion

		private void saveSettings_Click(object sender, EventArgs e) {
			settingsBox.ShowSettingsBox(0, motor.DeviceID);
		}
		private void revertToSavedSettings_Click(object sender, EventArgs e) {
			settingsBox.ShowSettingsBox(1, motor.DeviceID);
		}
		private void resetSettings_Click(object sender, EventArgs e) {
			settingsBox.ShowSettingsBox(2, motor.DeviceID);
		}
		#endregion

		#region additional motor settings
		private void showOtherSettings_CheckedChanged(object sender, EventArgs e) {
			CheckBox check = (CheckBox)sender;
			if (check.Checked) {
				motorSettingsBox.Visible = true;
				this.Size = new Size(this.Size.Width + motorSettingsBox.Width + 10, this.Size.Height);

			} else {
				this.Size = new Size(this.Size.Width - motorSettingsBox.Width, this.Size.Height);
				motorSettingsBox.Visible = false;
			}
		}

		#region current regulator gain
		private void currentRegText_ValueChanged(object sender, EventArgs e) {
			setCurrentRegulatorGain();
		}

		private void currentRegText_KeyDown(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				e.SuppressKeyPress = true;
				setCurrentRegulatorGain();
			}
		}

		private void setCurrentRegulatorGain() {
			try {
				motor.CurrentRegulatorGain = Convert.ToDouble(currentRegText.Value);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting current regulator gain " + ex.Message);
			}
		}
		#endregion

		#region current limit
		private void currentLimitText_ValueChanged(object sender, EventArgs e) {
			setCurrentLimit();
		}

		private void currentLimitText_KeyDown(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				e.SuppressKeyPress = true;
				setCurrentLimit();
			}
		}

		private void setCurrentLimit() {
			try {
				motor.CurrentLimit = Convert.ToDouble(currentLimitText.Value);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting current limit " + ex.Message);
			}
		}
		#endregion

		#region stall velocity
		private void stallVelocityText_ValueChanged(object sender, EventArgs e) {
			setStallVelocity();
		}

		private void stallVelocityText_KeyDown(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				e.SuppressKeyPress = true;
				setStallVelocity();
			}
		}

		private void setStallVelocity() {
			try {
				motor.StallVelocity = Convert.ToDouble(stallVelocityText.Value);
				stallVelocityText.Increment = updateLimits(motor.StallVelocity);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting stall velocity " + ex.Message);
			}
		}
		#endregion


		private void fanModeCombo_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				motor.FanMode = (FanMode)fanModeCombo.Items[fanModeCombo.SelectedIndex];
			} catch (Exception ex) {
				errorBox.addMessage("Error setting fan mode: " + ex.Message);
			}
		}

		private void encoderIOCombo_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				motor.IOMode = (EncoderIOMode)encoderIOCombo.Items[encoderIOCombo.SelectedIndex];
			} catch (Exception ex) {
				errorBox.addMessage("Error setting IO Mode: " + ex.Message);
			}
		}

		public void saveOrResetSettings(CheckedListBox.CheckedItemCollection collection, int state) {
			settingsBox.Hide();
			string outputText = "";
			if (state == 0) //save settings
			{
				outputText = "Saved settings!";
				for (int i = 0; i < collection.Count; i++) {
					if (collection[i].ToString() == "Kp") {
						Properties.Settings.Default.Kp = motor.Kp;
						Properties.Settings.Default.isKpSet = true;
					} else if (collection[i].ToString() == "Ki") {
						Properties.Settings.Default.Ki = motor.Ki;
						Properties.Settings.Default.isKiSet = true;
					} else if (collection[i].ToString() == "Kd") {
						Properties.Settings.Default.Kd = motor.Kd;
						Properties.Settings.Default.isKdSet = true;
					} else if (collection[i].ToString() == "Acceleration") {
						Properties.Settings.Default.Acceleration = motor.Acceleration;
						Properties.Settings.Default.isAccelerationSet = true;
					} else if (collection[i].ToString() == "Velocity Limit") {
						Properties.Settings.Default.VelocityLimit = motor.VelocityLimit;
						Properties.Settings.Default.isVelocityLimitSet = true;
					} else if (collection[i].ToString() == "Rescale Factor") {
						Properties.Settings.Default.rescaleFactor = motor.RescaleFactor;
						Properties.Settings.Default.isRescaleFactorSet = true;
					} else if (collection[i].ToString() == "Deadband") {
						Properties.Settings.Default.Deadband = motor.DeadBand;
						Properties.Settings.Default.isDeadbandSet = true;
					} else if (collection[i].ToString() == "Current Limit") {
						Properties.Settings.Default.CurrentLimit = motor.CurrentLimit;
						Properties.Settings.Default.isCurrentLimitSet = true;
					} else if (collection[i].ToString() == "Current Regulator Gain") {
						Properties.Settings.Default.CurrentRegulatorGain = motor.CurrentRegulatorGain;
						Properties.Settings.Default.isCurrentRegulatorGainSet = true;
					} else if (collection[i].ToString() == "Stall Velocity") {
						Properties.Settings.Default.StallVelocity = motor.StallVelocity;
						Properties.Settings.Default.isStallVelocitySet = true;
					} else if (collection[i].ToString() == "Encoder IO Mode") {
						Properties.Settings.Default.encoderIOMode = motor.IOMode;
						Properties.Settings.Default.isEncoderIOModeSet = true;
					}
				}
				Properties.Settings.Default.Save();
				updateGUI(motor);
			} else if (state == 1)//reverting to saved settings
			  {
				outputText = "Reverted to saved settings!";
				applySavedSettings(motor, collection);
			} else if (state == 2)//reverting to default settings
			  {
				outputText = "Reverted to default settings!";
				setDefaults(motor, collection);
			}

			if (collection.Count > 0)
				MessageBox.Show(outputText);
		}

		#endregion

		private void setDataPoints_Click(object sender, EventArgs e) {
			int result;
			if (int.TryParse(dataPointsText.Text, out result)) {
				if (result > 0) { //can't have negative increment value
					dataPoints = result;
					Properties.Settings.Default.dataPoints = result;
					Properties.Settings.Default.isDataPointsSet = true;
					Properties.Settings.Default.Save();
				}
			}
		}
	}
}
